package GData::Controller::EntryCollection;

use strict;
use warnings;
use base qw(Catalyst::Controller::Atompub::Collection);

use Search::Estraier;
sub get_feed :Atompub(list) {
    my($self, $c) = @_;

    # 検索条件オブジェクトを構築する
    my $cond = Search::Estraier::Condition->new;

    $cond->set_options(qw(SIMPLE));      # 簡便書式に設定
    $cond->add_attr('@updated NUMGE 0'); # updated が 0 以上
    $cond->set_order('@updated NUMD');   # updated の降順にソート

    # 全文検索クエリの解析
    if (my $q = $c->req->param('q')) {
        $q =~ s/\bOR\b/|/g;    # OR を | に置換する
        $q =~ s/\s+-/ ! /g;    # キーワード頭の - を ! に置換する
        $cond->set_phrase($q);
    }

    # カテゴリクエリの解析
    my @args;
    if ((@args = @{ $c->req->args }) > 2 && $args[1] eq '-') {
        $args[2] =~ s/\|/ /g ?
            $cond->add_attr("\@category ISTROR $args[2]")
          : $cond->add_attr('@category ISTRAND '.join(' ', @args[ 2..$#args ]));
    }

    # 著者クエリの解析
    if (my $author = $c->req->param('author')) {
        $cond->add_attr("\@author ISTRAND $author");
    }

    # 日時クエリの解析
    my @t = ([qw(updated   min NUMGE)], [qw(updated   max NUMLT)],
             [qw(published min NUMGE)], [qw(published max NUMLT)],);
    for (@t) {
        if (my $t = $c->req->param("$_->[0]-$_->[1]")) {
            $cond->add_attr(join ' ', '@'.$_->[0], $_->[2], $t);
        }
    }

    # 取得結果の開始位置と最大数の解析
    $cond->set_skip($c->req->param('start-index')-1)
        if $c->req->param('start-index');
    $cond->set_max($c->req->param('max-results'))
        if $c->req->param('max-results');

    # 検索の実行
    my $rs = $c->model('Entries')->search($cond, 0);

    # フィード (XML::Atom::Feed) のひな型
    my $feed = $self->collection_resource->body;

    # フィードのひな形にエントリを追加する
    for my $i (0 .. $rs->doc_num-1) {
        my $doc = $rs->get_doc($i);
        my $entry = XML::Atom::Entry->new(\$doc->attr('@xml'));
        $feed->add_entry($entry);
    }

    return 1;
}

sub do_update_or_create_entry :Atompub(create, update) {
    my($self, $c) = @_;

    # エントリ本体を取得する (XML::Atom::Entry)
    my $entry = $self->entry_resource->body;

    # エントリ本文
    my $xml = $entry->as_xml;
       $xml =~ s/[\r\n]+/ /g; # 改行文字を除去する

    # category 要素
    my $category = join ' ', map { '{'.($_->scheme || '').'}'.$_->term } $entry->category;

    # author の子要素
    my $author = join ' ', grep { defined } map { ($_->name, $_->email) } $entry->author;

    # 文書オブジェクトに属性と本文を設定する
    my $doc = Search::Estraier::Document->new;
       $doc->add_attr('@uri',       $self->entry_resource->uri);
       $doc->add_attr('@category',  $category) if $category;
       $doc->add_attr('@author',    $author) if $author;
       $doc->add_attr('@updated',   $entry->updated);
       $doc->add_attr('@published', $entry->published) if $entry->published;
       $doc->add_attr('@xml',       $xml);
       $doc->add_text($xml);

    # データベースに文書を追加する
    $c->model('Entries')->put_doc($doc);

    return 1;
}

sub get_entry :Atompub(read) {
    my($self, $c) = @_;

    # 検索条件オブジェクトを構築する
    my $cond = Search::Estraier::Condition->new;
       $cond->add_attr('@uri STREQ '.$c->req->uri); # URI が一致

    # 検索を実行する
    my $rs = $c->model('Search')->search($cond, 0);

    # エントリが見つからなければ 404 エラーを返す
    return $self->error($c, 404) unless $rs->doc_num;

    # エントリリソースをセットする
    my $doc = $rs->get_doc(0);
    my $entry = XML::Atom::Entry->new(\$doc->attr('@xml'));
    $self->entry_resource->body($entry);

    return 1;
}

sub delete_entry :Atompub(delete) {
    my($self, $c) = @_;
    $c->model('Search')->out_doc_by_uri($c->req->uri);
    return 1;
}

use Google::Data::JSON qw(gdata);
sub end :ActionClass('RenderView') {
    my($self, $c) = @_;

    # alt が指定されていなければ何もしない
    unless ($c->req->param('alt')) {
    }
    # alt が json であれば JSON に変換する
    elsif ($c->req->param('alt') eq 'json') {
        $c->res->body( gdata($c->res->body)->as_json );
        $c->res->content_type('application/json');
    }
    # alt が json-in-script であればコールバック関数をセットする
    elsif ($c->req->param('alt') eq 'json-in-script' && $c->req->param('callback')) {
        $c->res->body( $c->req->param('callback').'('.gdata($c->res->body)->as_json.')' );
        $c->res->content_type('application/json');
    }
}

# Make EditURI by yourself (it will be automatically generated by default)
#sub make_edit_uri {
#    my($self, $c, @args) = @_;
#}

# Find version information of the Resource for caching and versioning
#sub find_version {
#    my($self, $c, $uri) = @_;
#}

=head1 NAME

GData::Controller::EntryCollection - Atom Collection Resources

=head1 SYNOPSIS

See L<GData>.

=head1 DESCRIPTION

Atom Collection Resources class.

=head1 AUTHOR

Takeru INOUE

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
